@@ -81,6 +81,8 @@ typedef struct _Panel {
     guint setstrut : 1;
     guint round_corners : 1;
     guint usefontcolor : 1;
+    guint usefontsize : 1;
+    guint fontsize;
     guint transparent : 1;
     guint background : 1;
     guint spacing;
@@ -179,7 +181,9 @@ enum {
     CONF_TYPE_INT,
     CONF_TYPE_BOOL,
     CONF_TYPE_FILE,
-    CONF_TYPE_FILE_ENTRY
+    CONF_TYPE_FILE_ENTRY,
+    CONF_TYPE_DIRECTORY_ENTRY,
+    CONF_TYPE_TRIM
 };
 
 enum { LINE_NONE, LINE_BLOCK_START, LINE_BLOCK_END, LINE_VAR };
