@@ -21,7 +21,7 @@
 
 #include <menu-cache.h>
 
-extern MenuCache * panel_menu_cache_new(void);					/* Allocate a menu cache */
-extern gboolean panel_menu_item_evaluate_visibility(MenuCacheItem * item);	/* Evaluate the visibility of a menu item */
+extern MenuCache * panel_menu_cache_new(guint32* visibility_flags); /* Allocate a menu cache */
+extern gboolean panel_menu_item_evaluate_visibility(MenuCacheItem * item, guint32 visibility_flags); /* Evaluate the visibility of a menu item */
 
 #endif
