@@ -225,7 +225,8 @@ extern void panel_apply_icon(GtkWindow *w);
 extern void panel_destroy(Panel *p);
 extern void panel_adjust_geometry_terminology(Panel *p);
 extern void panel_determine_background_pixmap(Panel * p, GtkWidget * widget, GdkWindow * window);
-extern void panel_draw_label_text(Panel * p, GtkWidget * label, char * text, gboolean bold, gboolean custom_color);
+extern void panel_draw_label_text(Panel * p, GtkWidget * label, char * text, gboolean bold,
+        float custom_size_factor, gboolean custom_color);
 extern void panel_establish_autohide(Panel *p);
 extern void panel_image_set_from_file(Panel * p, GtkWidget * image, const char * file);
 extern gboolean panel_image_set_icon_theme(Panel * p, GtkWidget * image, const gchar * icon);
