@@ -67,7 +67,7 @@ typedef struct _task {
 typedef struct _desk {
     struct _pager * pg;				/* Back pointer to plugin context */
     GtkWidget * da;				/* Drawing area */
-    GdkPixmap * pixmap;				/* Pixmap to be drawn on drawing area */
+    cairo_surface_t * pixmap;			/* Pixmap to be drawn on drawing area */
     int desktop_number;				/* Desktop number */
     gboolean dirty;				/* True if needs to be recomputed */
     gfloat scale_x;				/* Horizontal scale factor */
@@ -212,24 +212,34 @@ static void task_update_pixmap(PagerTask * tk, PagerDesk * d)
         if ((tk->desktop == ALL_DESKTOPS) || (tk->desktop == d->desktop_number))
         {
             /* Scale the representation of the window to the drawing area. */
-            int x = (gfloat) tk->x * d->scale_x;
-            int y = (gfloat) tk->y * d->scale_y;
-            int w = (gfloat) tk->w * d->scale_x;
-            int h = ((tk->nws.shaded) ? 3 : (gfloat) tk->h * d->scale_y);
+            gfloat x = (gfloat) tk->x * d->scale_x;
+            gfloat y = (gfloat) tk->y * d->scale_y;
+            gfloat w = (gfloat) tk->w * d->scale_x;
+            gfloat h = ((tk->nws.shaded) ? 3 : (gfloat) tk->h * d->scale_y);
             if ((w >= 3) && (h >= 3))
             {
                 /* Draw the window representation and a border. */
                 GtkWidget * widget = GTK_WIDGET(d->da);
                 GtkStyle * style = gtk_widget_get_style(widget);
+                GdkColor * color;
 
-                gdk_draw_rectangle(d->pixmap,
-                    (d->pg->focused_task == tk) ? style->bg_gc[GTK_STATE_SELECTED] : style->bg_gc[GTK_STATE_NORMAL],
-                    TRUE,
-                    x + 1, y + 1, w - 1, h - 1);
-                gdk_draw_rectangle(d->pixmap,
-                    (d->pg->focused_task == tk) ? style->fg_gc[GTK_STATE_SELECTED] : style->fg_gc[GTK_STATE_NORMAL],
-                    FALSE,
-                    x, y, w, h);
+                cairo_t * cr = cairo_create(d->pixmap);
+                cairo_set_line_width (cr, 1.0);
+		cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
+                cairo_rectangle(cr, x + 0.5, y + 0.5, w - 0.5, h - 0.5);
+
+                color =
+                    (d->pg->focused_task == tk) ? &style->bg[GTK_STATE_SELECTED] : &style->bg[GTK_STATE_NORMAL];
+                cairo_set_source_rgb(cr, (double)color->red/65535, (double)color->green/65535, (double)color->blue/65535);
+                cairo_fill_preserve(cr);
+
+                color =
+                    (d->pg->focused_task == tk) ? &style->fg[GTK_STATE_SELECTED] : &style->fg[GTK_STATE_NORMAL];
+                cairo_set_source_rgb(cr, (double)color->red/65535, (double)color->green/65535, (double)color->blue/65535);
+                cairo_stroke(cr);
+
+		check_cairo_status(cr);
+                cairo_destroy(cr);
             }
         }
     }
@@ -271,10 +281,10 @@ static gboolean desk_configure_event(GtkWidget * widget, GdkEventConfigure * eve
 {
     /* Allocate pixmap and statistics buffer without border pixels. */
 #if GTK_CHECK_VERSION(2,18,0)
-    GtkAllocation allocation;
-    gtk_widget_get_allocation(widget, &allocation);
-    int new_pixmap_width = allocation.width;
-    int new_pixmap_height = allocation.height;
+    GtkAllocation *allocation = g_new0 (GtkAllocation, 1);
+    gtk_widget_get_allocation(GTK_WIDGET(widget), allocation);
+    int new_pixmap_width = allocation->width;
+    int new_pixmap_height = allocation->height;
 #else
     int new_pixmap_width = widget->allocation.width;
     int new_pixmap_height = widget->allocation.height;
@@ -283,17 +293,19 @@ static gboolean desk_configure_event(GtkWidget * widget, GdkEventConfigure * eve
     {
         /* Allocate a new pixmap of the allocated size. */
         if (d->pixmap != NULL)
-            g_object_unref(d->pixmap);
-#if GTK_CHECK_VERSION(2,14,0)
-        d->pixmap = gdk_pixmap_new(gtk_widget_get_window(widget), new_pixmap_width, new_pixmap_height, -1);
-#else
-        d->pixmap = gdk_pixmap_new(widget->window, new_pixmap_width, new_pixmap_height, -1);
-#endif
+            cairo_surface_destroy(d->pixmap);
+        d->pixmap = cairo_image_surface_create(CAIRO_FORMAT_RGB24, new_pixmap_width, new_pixmap_height);
+        cairo_t *cr = cairo_create(d->pixmap);
+        cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
+        cairo_paint(cr);
+	check_cairo_status(cr);
+        cairo_destroy(cr);
+	check_cairo_surface_status(&d->pixmap);
 
         /* Compute the horizontal and vertical scale factors, and mark the desktop for redraw. */
 #if GTK_CHECK_VERSION(2,18,0)
-        d->scale_y = (gfloat) allocation.height / (gfloat) gdk_screen_height();
-        d->scale_x = (gfloat) allocation.width  / (gfloat) gdk_screen_width();
+        d->scale_y = (gfloat) allocation->height / (gfloat) gdk_screen_height();
+        d->scale_x = (gfloat) allocation->width  / (gfloat) gdk_screen_width();
 #else
         d->scale_y = (gfloat) allocation->height / (gfloat) gdk_screen_height();
         d->scale_x = (gfloat) allocation->width  / (gfloat) gdk_screen_width();
@@ -305,6 +317,9 @@ static gboolean desk_configure_event(GtkWidget * widget, GdkEventConfigure * eve
     gtk_widget_set_size_request(widget,
         (d->pg->plugin->panel->icon_size - BORDER_WIDTH * 2) * d->pg->aspect_ratio,
         d->pg->plugin->panel->icon_size - BORDER_WIDTH * 2);
+#if GTK_CHECK_VERSION(2,18,0)
+    g_free (allocation);
+#endif
     return FALSE;
 }
 
@@ -315,6 +330,9 @@ static gboolean desk_expose_event(GtkWidget * widget, GdkEventExpose * event, Pa
 
     if (d->pixmap != NULL)
     {
+        cairo_t *cr = gdk_cairo_create(gtk_widget_get_window(widget));
+        gdk_cairo_region(cr, event->region);
+        cairo_clip(cr);
         /* Recompute the pixmap if needed. */
         if (d->dirty)
         {
@@ -324,22 +342,15 @@ static gboolean desk_expose_event(GtkWidget * widget, GdkEventExpose * event, Pa
             /* Erase the pixmap. */
             if (d->pixmap != NULL)
             {
-                GtkWidget * widget = GTK_WIDGET(d->da);
-#if GTK_CHECK_VERSION(2,18,0)
-                GtkAllocation allocation;
-                gtk_widget_get_allocation(widget, &allocation);
-#endif
-                gdk_draw_rectangle(
-                    d->pixmap,
+                //GtkWidget * widget = GTK_WIDGET(d->da);
+                cairo_t *cr0 = cairo_create(d->pixmap);
+                gdk_cairo_set_source_color(cr0,
                     ((d->desktop_number == d->pg->current_desktop)
-                        ? style->dark_gc[GTK_STATE_SELECTED]
-                        : style->dark_gc[GTK_STATE_NORMAL]),
-                    TRUE,
-#if GTK_CHECK_VERSION(2,18,0)
-                    0, 0, allocation.width, allocation.height);
-#else
-                    0, 0, widget->allocation.width, widget->allocation.height);
-#endif
+                        ? &style->dark[GTK_STATE_SELECTED]
+                        : &style->dark[GTK_STATE_NORMAL]));
+                cairo_paint(cr0);
+		check_cairo_status(cr0);
+                cairo_destroy(cr0);
             }
 
             /* Draw tasks onto the pixmap. */
@@ -349,16 +360,14 @@ static gboolean desk_expose_event(GtkWidget * widget, GdkEventExpose * event, Pa
         }
 
         /* Draw the requested part of the pixmap onto the drawing area. */
-#if GTK_CHECK_VERSION(2,14,0)
-        gdk_draw_drawable(gtk_widget_get_window(widget), 
-#else
-        gdk_draw_drawable(widget->window,
-#endif
-              style->fg_gc[GTK_WIDGET_STATE(widget)],
-              d->pixmap,
-              event->area.x, event->area.y,
-              event->area.x, event->area.y,
-              event->area.width, event->area.height);
+        GtkAllocation allocation;
+        gtk_widget_get_allocation(GTK_WIDGET(widget), &allocation);
+        gdk_cairo_set_source_color(cr,
+             &style->fg[GTK_WIDGET_STATE(widget)]);
+        cairo_set_source_surface(cr, d->pixmap, 0, 0);
+        cairo_paint(cr);
+	check_cairo_status(cr);
+        cairo_destroy(cr);
     }
     return FALSE;
 }
@@ -436,7 +445,7 @@ static void desk_free(PagerPlugin * pg, int desktop_number)
     icon_grid_remove(pg->icon_grid, d->da);
 
     if (d->pixmap != NULL)
-        g_object_unref(d->pixmap);
+        cairo_surface_destroy(d->pixmap);
 
     g_free(d);
 }
