@@ -197,6 +197,7 @@ static void taskbar_panel_configuration_changed(Plugin * p);
 static void set_timer_on_task(Task * tk)
 {
     gint interval;
+    g_return_if_fail(tk->flash_timeout == 0);
     g_object_get(gtk_widget_get_settings(tk->button), "gtk-cursor-blink-time", &interval, NULL);
     tk->flash_timeout = g_timeout_add(interval, (GSourceFunc) flash_window_timeout, tk);
 }
@@ -296,20 +297,25 @@ static void task_draw_label(Task * tk)
 {
     TaskClass * tc = tk->res_class;
     gboolean bold_style = (((tk->entered_state) || (tk->flash_state)) && (tk->tb->flat_button));
-    if ((tk->tb->grouped_tasks) && (tc != NULL) && (tc->visible_task == tk) && (tc->visible_count > 1))
-	{
-        char * label = g_strdup_printf("(%d) %s", tc->visible_count, tc->visible_name);
-        gtk_widget_set_tooltip_text(tk->button, label);
-        panel_draw_label_text(tk->tb->plug->panel, tk->label, label, bold_style, tk->tb->flat_button);
-        g_free(label);
-	}
+    char *label;
+
+    if ((tk->tb->grouped_tasks) && (tc != NULL) && (tc->visible_task == tk)
+            && (tc->visible_count > 1))
+    {
+        label = g_strdup_printf("(%d) %s", tc->visible_count, tc->visible_name);
+    }
     else
     {
-        char * name = tk->iconified ? tk->name_iconified : tk->name;
-        if (tk->tb->tooltips)
-            gtk_widget_set_tooltip_text(tk->button, name);
-        panel_draw_label_text(tk->tb->plug->panel, tk->label, name, bold_style, tk->tb->flat_button);
+        label = g_strdup(tk->iconified ? tk->name_iconified : tk->name);
     }
+
+    if (tk->tb->tooltips)
+        gtk_widget_set_tooltip_text(tk->button, label);
+
+    panel_draw_label_text(tk->tb->plug->panel, tk->label, label, bold_style, 1,
+            tk->tb->flat_button);
+
+    g_free(label);
 }
 
 /* Determine if a task is visible. */
@@ -2061,3 +2067,5 @@ PluginClass taskbar_plugin_class = {
     panel_configuration_changed : taskbar_panel_configuration_changed
 
 };
+
+/* vim: set sw=4 et sts=4 : */
