@@ -30,8 +30,10 @@
 #include "plugin.h"
 #include "dbg.h"
 
-#define ICONS_VOLUME PACKAGE_DATA_DIR "/lxpanel/images/volume.png"
-#define ICONS_MUTE PACKAGE_DATA_DIR "/lxpanel/images/mute.png"
+#define ICONS_VOLUME_HIGH   PACKAGE_DATA_DIR "/lxpanel/images/volume-high.png"
+#define ICONS_VOLUME_MEDIUM PACKAGE_DATA_DIR "/lxpanel/images/volume-medium.png"
+#define ICONS_VOLUME_LOW    PACKAGE_DATA_DIR "/lxpanel/images/volume-low.png"
+#define ICONS_MUTE          PACKAGE_DATA_DIR "/lxpanel/images/mute.png"
 
 typedef struct {
 
@@ -50,12 +52,24 @@ typedef struct {
     snd_mixer_selem_id_t * sid;			/* The element ID */
     snd_mixer_elem_t * master_element;		/* The Master element */
     guint mixer_evt_idle;			/* Timer to handle restarting poll */
+
+    /* unloading and error handling */
+    GIOChannel **channels;                      /* Channels that we listen to */
+    guint num_channels;                         /* Number of channels */
+
+    /* Icons */
+    const char* icon;
+    const char* icon_panel;
+    const char* icon_fallback;
+
 } VolumeALSAPlugin;
 
 static gboolean asound_find_element(VolumeALSAPlugin * vol, const char * ename);
 static gboolean asound_reset_mixer_evt_idle(VolumeALSAPlugin * vol);
 static gboolean asound_mixer_event(GIOChannel * channel, GIOCondition cond, gpointer vol_gpointer);
+static gboolean asound_restart(gpointer vol_gpointer);
 static gboolean asound_initialize(VolumeALSAPlugin * vol);
+static void asound_deinitialize(VolumeALSAPlugin * vol);
 static gboolean asound_has_mute(VolumeALSAPlugin * vol);
 static gboolean asound_is_muted(VolumeALSAPlugin * vol);
 static int asound_get_volume(VolumeALSAPlugin * vol);
@@ -120,11 +134,12 @@ static gboolean asound_reset_mixer_evt_idle(VolumeALSAPlugin * vol)
 static gboolean asound_mixer_event(GIOChannel * channel, GIOCondition cond, gpointer vol_gpointer)
 {
     VolumeALSAPlugin * vol = (VolumeALSAPlugin *) vol_gpointer;
+    int res = 0;
 
     if (vol->mixer_evt_idle == 0)
     {
         vol->mixer_evt_idle = g_idle_add_full(G_PRIORITY_DEFAULT, (GSourceFunc) asound_reset_mixer_evt_idle, vol, NULL);
-        snd_mixer_handle_events(vol->mixer);
+        res = snd_mixer_handle_events(vol->mixer);
     }
 
     if (cond & G_IO_IN)
@@ -133,15 +148,40 @@ static gboolean asound_mixer_event(GIOChannel * channel, GIOCondition cond, gpoi
         volumealsa_update_display(vol);
     }
 
-    if (cond & G_IO_HUP)
+    if ((cond & G_IO_HUP) || (res < 0))
     {
         /* This means there're some problems with alsa. */
+	ERR("volumealsa: ALSA (or pulseaudio) had a problem: \n"
+                "volumealsa: snd_mixer_handle_events() = %d,"
+                " cond 0x%x (IN: 0x%x, HUP: 0x%x).\n", res, cond,
+                G_IO_IN, G_IO_HUP);
+        gtk_widget_set_tooltip_text(vol->plugin->pwid, "ALSA (or pulseaudio) had a problem."
+                " Please check the lxpanel logs.");
+
+        g_timeout_add_seconds(1, asound_restart, vol);
+
         return FALSE;
     }
 
     return TRUE;
 }
 
+static gboolean asound_restart(gpointer vol_gpointer)
+{
+    VolumeALSAPlugin * vol = vol_gpointer;
+
+    asound_deinitialize(vol);
+
+    if (!asound_initialize(vol)) {
+        ERR("volumealsa: Re-initialization failed.\n");
+        return TRUE; // try again in a second
+    }
+
+    ERR("volumealsa: Restarted ALSA interface...\n");
+
+    return FALSE;
+}
+
 /* Initialize the ALSA interface. */
 static gboolean asound_initialize(VolumeALSAPlugin * vol)
 {
@@ -167,18 +207,42 @@ static gboolean asound_initialize(VolumeALSAPlugin * vol)
     int n_fds = snd_mixer_poll_descriptors_count(vol->mixer);
     struct pollfd * fds = g_new0(struct pollfd, n_fds);
 
+    vol->channels = g_new0(GIOChannel *, n_fds);
+    vol->num_channels = n_fds;
+
     snd_mixer_poll_descriptors(vol->mixer, fds, n_fds);
     int i;
     for (i = 0; i < n_fds; ++i)
     {
         GIOChannel* channel = g_io_channel_unix_new(fds[i].fd);
         g_io_add_watch(channel, G_IO_IN | G_IO_HUP, asound_mixer_event, vol);
-        g_io_channel_unref(channel);
+        vol->channels[i] = channel;
     }
     g_free(fds);
     return TRUE;
 }
 
+static void asound_deinitialize(VolumeALSAPlugin * vol)
+{
+    int i;
+
+    if (vol->mixer_evt_idle != 0) {
+        g_source_remove(vol->mixer_evt_idle);
+        vol->mixer_evt_idle = 0;
+    }
+
+    for (i = 0; i < vol->num_channels; i++) {
+        g_io_channel_shutdown(vol->channels[i], FALSE, NULL);
+        g_io_channel_unref(vol->channels[i]);
+    }
+    g_free(vol->channels);
+    vol->channels = NULL;
+    vol->num_channels = 0;
+
+    snd_mixer_close(vol->mixer);
+    vol->master_element = NULL;
+}
+
 /* Get the presence of the mute control from the sound system. */
 static gboolean asound_has_mute(VolumeALSAPlugin * vol)
 {
@@ -223,15 +287,62 @@ static void asound_set_volume(VolumeALSAPlugin * vol, int volume)
 
 /*** Graphics ***/
 
+static void volumealsa_update_current_icon(VolumeALSAPlugin * vol)
+{
+    /* Mute status. */
+    gboolean mute = asound_is_muted(vol);
+    int level = asound_get_volume(vol);
+    
+    /* Change icon according to mute / volume */
+    const char* icon="audio-volume-muted";
+    const char* icon_panel="audio-volume-muted-panel";
+    const char* icon_fallback=ICONS_MUTE;
+    if (mute)
+    {
+         icon_panel = "audio-volume-muted-panel";
+         icon="audio-volume-muted";
+         icon_fallback=ICONS_MUTE;
+    }
+    else if (level >= 75)
+    {
+         icon_panel = "audio-volume-high-panel";
+         icon="audio-volume-high";
+         icon_fallback=ICONS_VOLUME_HIGH;
+    }
+    else if (level >= 50)
+    {
+         icon_panel = "audio-volume-medium-panel";
+         icon="audio-volume-medium";
+         icon_fallback=ICONS_VOLUME_MEDIUM;
+    }
+    else if (level > 0)
+    {
+         icon_panel = "audio-volume-low-panel";
+         icon="audio-volume-low";
+         icon_fallback=ICONS_VOLUME_LOW;
+    }
+
+    vol->icon_panel = icon_panel;
+    vol->icon = icon;
+    vol->icon_fallback= icon_fallback;
+}
+
 /* Do a full redraw of the display. */
 static void volumealsa_update_display(VolumeALSAPlugin * vol)
 {
     /* Mute status. */
     gboolean mute = asound_is_muted(vol);
-    
-    if ( ! panel_image_set_icon_theme(vol->plugin->panel, vol->tray_icon, ((mute) ? "audio-volume-muted" : "audio-volume-high")))
+    int level = asound_get_volume(vol);
+
+    volumealsa_update_current_icon(vol);
+
+    /* Change icon, fallback to default icon if theme doesn't exsit */
+    if ( ! panel_image_set_icon_theme(vol->plugin->panel, vol->tray_icon, vol->icon_panel))
     {
-         panel_image_set_from_file(vol->plugin->panel, vol->tray_icon, ((mute) ? ICONS_MUTE : ICONS_VOLUME));
+        if ( ! panel_image_set_icon_theme(vol->plugin->panel, vol->tray_icon, vol->icon))
+        {
+            panel_image_set_from_file(vol->plugin->panel, vol->tray_icon, vol->icon_fallback);
+        }
     }
 
     g_signal_handler_block(vol->mute_check, vol->mute_check_handler);
@@ -240,7 +351,6 @@ static void volumealsa_update_display(VolumeALSAPlugin * vol)
     g_signal_handler_unblock(vol->mute_check, vol->mute_check_handler);
 
     /* Volume. */
-    int level = asound_get_volume(vol);
     if (vol->volume_scale != NULL)
     {
         g_signal_handler_block(vol->volume_scale, vol->volume_scale_handler);
@@ -254,6 +364,7 @@ static void volumealsa_update_display(VolumeALSAPlugin * vol)
     g_free(tooltip);
 }
 
+
 /* Handler for "button-press-event" signal on main widget. */
 static gboolean volumealsa_button_press_event(GtkWidget * widget, GdkEventButton * event, VolumeALSAPlugin * vol)
 {
@@ -299,6 +410,17 @@ static void volumealsa_popup_map(GtkWidget * widget, VolumeALSAPlugin * vol)
     plugin_adjust_popup_position(widget, vol->plugin);
 }
 
+static void volumealsa_theme_change(GtkWidget * widget, VolumeALSAPlugin * vol)
+{
+    if ( ! panel_image_set_icon_theme(vol->plugin->panel, vol->tray_icon, vol->icon_panel))
+    {
+        if ( ! panel_image_set_icon_theme(vol->plugin->panel, vol->tray_icon, vol->icon))
+        {
+            panel_image_set_from_file(vol->plugin->panel, vol->tray_icon, vol->icon_fallback);
+        }
+    }
+}
+
 /* Handler for "value_changed" signal on popup window vertical scale. */
 static void volumealsa_popup_scale_changed(GtkRange * range, VolumeALSAPlugin * vol)
 {
@@ -431,6 +553,7 @@ static int volumealsa_constructor(Plugin * p, char ** fp)
     /* Connect signals. */
     g_signal_connect(G_OBJECT(p->pwid), "button-press-event", G_CALLBACK(volumealsa_button_press_event), vol);
     g_signal_connect(G_OBJECT(p->pwid), "scroll-event", G_CALLBACK(volumealsa_popup_scale_scrolled), vol );
+    g_signal_connect(p->panel->icon_theme, "changed", G_CALLBACK(volumealsa_theme_change), vol );
 
     /* Update the display, show the widget, and return. */
     volumealsa_update_display(vol);
@@ -443,9 +566,7 @@ static void volumealsa_destructor(Plugin * p)
 {
     VolumeALSAPlugin * vol = (VolumeALSAPlugin *) p->priv;
 
-    /* Remove the periodic timer. */
-    if (vol->mixer_evt_idle != 0)
-        g_source_remove(vol->mixer_evt_idle);
+    asound_deinitialize(vol);
 
     /* If the dialog box is open, dismiss it. */
     if (vol->popup_window != NULL)
@@ -455,6 +576,79 @@ static void volumealsa_destructor(Plugin * p)
     g_free(vol);
 }
 
+/* Callback when the configuration dialog is to be shown. */
+
+static void volumealsa_configure(Plugin * p, GtkWindow * parent)
+{
+
+    GdkScreen *screen = gdk_screen_get_default();
+    GError *error = NULL;
+    const gchar *command_line = NULL;
+
+    if (g_find_program_in_path("pulseaudio"))
+    {
+     /* Assume that when pulseaudio is installed, it's launching every time */
+        if (g_find_program_in_path("gnome-sound-applet"))
+        {
+            command_line = "gnome-sound-applet";
+        }
+        else
+        {
+            if (g_find_program_in_path("pavucontrol"))
+            {
+                command_line = "pavucontrol";
+            }
+        }
+    }
+
+    /* Fallback to alsamixer when PA is not running, or when no PA utility is find */
+    if (command_line == NULL)
+    {
+        if (g_find_program_in_path("gnome-alsamixer"))
+        {
+            command_line = "gnome-alsamixer";
+        }
+        else
+        {
+            if (g_find_program_in_path("alsamixer"))
+            {
+                if (g_find_program_in_path("xterm"))
+                {
+                    command_line = "xterm -e alsamixer";
+                }
+            }
+        }
+    }
+
+    if (command_line)
+    {
+        gdk_spawn_command_line_on_screen(screen,
+                                         command_line,
+                                         &error);
+    }
+    else
+    {
+
+        GtkWidget* msg;
+
+        msg = gtk_message_dialog_new( NULL,
+                                      0,
+                                      GTK_MESSAGE_ERROR,
+                                      GTK_BUTTONS_OK,
+                                      (_("Error, you need to install an application to configure the sound (pavucontol, alsamixer ...)")) );
+        gtk_dialog_run( GTK_DIALOG(msg) );
+        gtk_widget_destroy( msg );
+
+    }
+
+    if (error)
+    {
+        g_print("%s\n", error->message);
+        g_free (error);
+    }
+
+}
+
 /* Callback when panel configuration changes. */
 static void volumealsa_panel_configuration_changed(Plugin * p)
 {
@@ -470,12 +664,14 @@ PluginClass volumealsa_plugin_class = {
     type : "volumealsa",
     name : N_("Volume Control"),
     version: "1.0",
-    description : "Display and control volume for ALSA",
+    description : N_("Display and control volume for ALSA"),
 
     constructor : volumealsa_constructor,
     destructor  : volumealsa_destructor,
-    config : NULL,
+    config :volumealsa_configure,
     save : NULL,
     panel_configuration_changed : volumealsa_panel_configuration_changed
 
 };
+
+/* vim: set sw=4 et sts=4 : */
